//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SasTasks.cc	--- implementation of SasTasks classes
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 1999-12-06: 1.0.0: - first version
//		UL, 2000-05-03: 1.1.0: - set 'imagebinning' parameter of evselect
//								 explicitely
//
//	AUTHOR:
//		U. Lammers, Mon Dec  6 14:03:43 MET 1999
//
//	$Header: /home/sasbuild/repositories/sasdev/sas/packages/srcdisplay/src/SasTasks.cc,v 1.5 2003/05/23 15:39:02 ulammers Exp $
//
#include <fstream>
#include <sstream>

#include "Dal.h"
#include "Task.h"
#include "common.h"
#include "param.h"
#include "ParamInfoSet.h"
#include "SasTasks.h"
#include "attcalc.h"
#include "atthkgen.h"
#include "evselect.h"
#include "srcdisplay.h"
#include "eboxdetect.h"
#include "imgdisplay.h"

using namespace std;

Evselect SasTasks::evselect;

void Evselect::image(const string &tableName,
					 const string &imageName,
					 const string &xAxis, const string &yAxis,
					 unsigned binX, unsigned binY)
{
	Info::Task *evselect = task(name().c_str());
	//	bin the event list into an image with evselect
	evselect->setParameter("table", tableName);
	evselect->setParameter("keepfilteroutput", false);
	evselect->setParameter("withimageset", true);
	evselect->setParameter("imageset", imageName);
	evselect->setParameter("withimagedatatype", true); // eboxdetect needs
													   // int32 images
	evselect->setParameter("imagedatatype", "Int32");
	evselect->setParameter("xcolumn", xAxis);
	evselect->setParameter("ycolumn", yAxis);
	evselect->setParameter("imagebinning", "binSize");
	evselect->setParameter("ximagebinsize", DBL(binX));
	evselect->setParameter("yimagebinsize", DBL(binY));

	evselect->exec();
} // Evselect::image()


Eboxdetect SasTasks::eboxdetect;

void Eboxdetect::detect(const string &imageSetName, const string &boxSetName,
					    unsigned nruns, double likemin)
{
	//	run eboxdetect
	Info::Task *eboxdetect = task(name().c_str());
	vector<string> is;
	is.push_back(imageSetName);
	eboxdetect->setParameter("imagesets", is);
	eboxdetect->setParameter("boxlistset", boxSetName);
	eboxdetect->setParameter("boxsize", 5);
	eboxdetect->setParameter("nruns", INT(nruns));
	eboxdetect->setParameter("likemin", likemin);
	eboxdetect->setParameter("usematchedfilter", false);
	eboxdetect->setParameter("usemap", false);
	eboxdetect->setParameter("withdetmask", false);
	eboxdetect->setParameter("withexpimage", false);

	eboxdetect->exec(true);	//	run via system

	//	check if we found any sources
	if (!dataSetServer->exists(boxSetName))
		errstr<<"eboxdetect found no sources in image `"<<imageSetName
			  <<"' - try to decrease detection likelihood"
			  <<Sas::error("NoSourcesFound");
} // Eboxdetect::detect()


Srcdisplay	SasTasks::srcdisplay;

void Srcdisplay::show(const string &imageSetName, const string &boxSetName,
					  double sourceRadius, bool includeSources,
					  bool withLabels, bool overlay,
					  ShapeBasics::ShapeColor color)
{
	Info::Task *srcdisplay = task(name().c_str());
	srcdisplay->setParameter("boxlistset", boxSetName);
	srcdisplay->setParameter("withimageset", !imageSetName.empty());
	if (!imageSetName.empty())
		srcdisplay->setParameter("imageset", imageSetName);
	srcdisplay->setParameter("sourceradius", sourceRadius);
	srcdisplay->setParameter("includesources", includeSources);
	srcdisplay->setParameter("uselabel", withLabels);
	srcdisplay->setParameter("overlay", overlay);
	srcdisplay->setParameter("srccolor", ShapeBasics::toString(color));

	srcdisplay->exec();
} // Srcdisplay::show()


Imgdisplay SasTasks::imgdisplay;

vector<string> Imgdisplay::getRegions()
{
	Info::Task *imgdisplay = task(name().c_str());
	//	get regions from imgdisplay
	imgdisplay->setParameter("withimagefile", false);
	imgdisplay->setParameter("withregion", true);
	imgdisplay->setParameter("withcommands", false);

	imgdisplay->exec();

	return imgdisplay->parameters()->parameter("regions")->stringListValue();
} // Imgdisplay::getRegions()

Attcalc SasTasks::attcalc;

void Attcalc::run(const string &eventListName)
{
	Info::Task *atthkgen = task("atthkgen");
	string atthkset =
			atthkgen->parameters()->parameter("atthkset")->stringValue();
	if (!dataSetServer->exists(atthkset))
		//	does not exist - create it
		atthkgen->exec();

	Info::Task *attcalc = task(name().c_str());
	attcalc->setParameter("eventset", eventListName);
	attcalc->setParameter("attitudelabel", "ahf");
	attcalc->setParameter("refpointlabel", "pnt");
	attcalc->setParameter("atthkset", atthkset);
	attcalc->exec(true);
} // Attcalc::run()

