//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SasTasks.h	--- I/F to Sas tasks
//
//	DESCRIPTION:
//		A task object represents a runnable SAS task.
//
//	CHANGE HISTORY:
//		UL, 1999-12-06: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Mon Dec  6 14:03:43 MET 1999
//
//	$Header: /home/sasbuild/repositories/sasdev/sas/packages/srcdisplay/src/SasTasks.h,v 1.5 2003/05/23 15:39:02 ulammers Exp $
//

#ifndef TASK_H
#define TASK_H

#include <vector>
#include <string>

#include "Region.h"

class SasTask {
public:
	SasTask(const std::string &name) { _name = name; }
	const std::string &name() const { return _name; }
	virtual void run() const { errstr<<"Do not know how to run `"
										  <<_name<<"'"
										  <<Sas::error("NotRunnable");
							   }
	virtual ~SasTask() { }

private:
	std::string	_name;
};

class Evselect : public SasTask {
public:
	Evselect() : SasTask("evselect") {}

	void image(const std::string &tableName,
			   const std::string &imageName,
			   const std::string &xAxis, const std::string &yAxis,
			   unsigned binX, unsigned binY);
		//
		//	PURPOSE:
		//		Accumulate an image from an event list with evselect
		//
		//	ARGUMENTS:
		//		tableName	: name of event list table
		//		imageName	: name of image set to create
		//		xAxis/yAxis	: column names in event list from which to
		//					  accumulate the image
		//		binX/binY	: bin sizes in X/Y
		//

	virtual ~Evselect() {}
};


class Eboxdetect : public SasTask {
public:
	Eboxdetect() : SasTask("eboxdetect") {}

	void detect(const std::string &imageSetName, const std::string &boxSetName,
				unsigned nruns = 1, double likemin = 15.);

	virtual ~Eboxdetect() {}
};



class Srcdisplay : public SasTask {
public:
	Srcdisplay() : SasTask("srcdisplay") {}

	void show(const std::string &imageSetName, const std::string &boxSetName,
			  double sourceRadius = .01, bool includeSources = true,
			  bool withLabels = false, bool overlay = false,
			  ShapeBasics::ShapeColor color = ShapeBasics::WHITE);

	virtual ~Srcdisplay() {}
};


class Imgdisplay : public SasTask {
public:
	Imgdisplay() : SasTask("imgdisplay") {}

	std::vector<std::string> getRegions();

	virtual ~Imgdisplay() {}
};


class Attcalc : public SasTask {
public:
	Attcalc() : SasTask("attcalc") {}

	void run(const std::string &eventListName);
		//
		//	PURPOSE:
		//		Run attcalc with 'attitudelabel=ahf'/'refpointlabel=pnt'
		//
		//	ARGUMENTS:
		//		eventListName	: name of event list
		//

	virtual ~Attcalc() {}
};


class SasTasks {
public:
	static Evselect		evselect;
	static Eboxdetect	eboxdetect;
	static Srcdisplay	srcdisplay;
	static Imgdisplay	imgdisplay;
	static Attcalc		attcalc;
};

#endif
