//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SourceList.h	--- a list of celestial sources
//
//	DESCRIPTION:
//		A SourceList object holds a list of celestial target.
//
//	CHANGE HISTORY:
//		UL, 1999-12-02: 1.0.0: - first version
//		UL, 2003-05-23: 2.0.0: - added new SourceList constructor to
//								 support new input mode of epicbscalgen
//
//	AUTHOR:
//		U. Lammers, Thu Dec  2 17:22:32 MET 1999
//
//	$Header: /home/sasbuild/repositories/sasdev/sas/packages/srcdisplay/src/SourceList.h,v 1.7 2003/05/23 15:39:02 ulammers Exp $
//
#ifndef SOURCE_LIST_H
#define SOURCE_LIST_H

#include <vector>
#include <string>
#include <iostream>

#include "Dal.h"
#include "Region.h"
#include "Celestial.h"
#include "Xmm.h"

class Source {
public:
	Source(double ra, double dec, FShape *shape, unsigned counts,
		   unsigned boxsize, double raDecErr = 0.)
		: _ra(ra), _dec(dec), _shape(shape), _counts(counts),
		  _boxsize(boxsize), _raDecErr(raDecErr) {}

	double ra() const { return _ra; }
	double dec() const { return _dec; }
	double raDecErr() const { return _raDecErr; }
	FShape *shape() const { return _shape; }
	unsigned counts() const { return _counts; }
	void counts(unsigned nr) { _counts = nr; }
	unsigned boxsize() { return _boxsize; }

	double angularSeparation(const Source &s) const;
		//
		//	PURPOSE:
		//		Return angular separation from other source in arcsecs
		//

	~Source() {}

private:
	double			_ra, _dec;		//	FK5 coordinates
	FShape			*_shape;		//	region around source
	unsigned		_counts,		//	number of counts in
					_boxsize;		//	boxsize
	double			_raDecErr;		//	error associated with _ra/_dec
};

std::ostream &operator<<(std::ostream &os, const Source &src);

class SourceList : public std::vector<Source> {
public:
	SourceList() {}
		//
		//	PURPOSE:
		//		Construct empty source list
		//


	SourceList(Table *sourceTable, double sourceRadius = 5,
			   bool includeSources = true,
			   ShapeBasics::ShapeColor color = ShapeBasics::WHITE);
	SourceList(const std::string &boxListSet, 
			   double sourceRadius = 5,
			   bool includeSources = true, 
			   bool uselabel=true,
			   ShapeBasics::ShapeColor color = ShapeBasics::WHITE);
		//
		//	PURPOSE:
		//		Construct source list from contents of source table
		//		as produced by eboxdetect task
		//
		//	ARGUMENTS:
		//		sourceTable	: pointer to table containing the source list
		//		sourceRadius: will put a circle region around each source
		//					  with this radius in image pixels
		//		includeSources: mark sources as 'included'/'excluded'
		//		uselabel	: label each source marker with index number?
		//		color		: source marker color
		//


	SourceList(const std::string &eventListName,
			   const std::string &xAxis, const std::string &yAxis,
			   const std::string &bsCcfName,
			   bool withBsCcfName = true,
			   unsigned binX = 75, unsigned binY = 75,
			   double sourceRadius = 5, unsigned nruns = 1,
			   double likemin = 15., bool runShowDetect = true);
		//
		//	PURPOSE:
		//		Construct a source list from an event list by
		//			a) running attcalc on the event list to compute
		//			   new sky coordinate columns X/Y
		//			b) accumulate an image with evselect
		//			c) run eboxdetect on the image
		//
		//	ARGUMENTS:
		//		eventListName	: name of event list for which to compute
		//						  new sky coordinate columns X/Y
		//		xAxis			: name of X axis
		//		yAxis			: name of Y axis
		//		BSeulerAngles	: Euler angles descibing the boresight
		//						  misalignment (3-2-1) according to SIAM ICD
		//		binX/binY		: bin sized in X/Y
		//		sourceRadius	: radius of circle to put around each found
		//						  source
		//		nruns			: number of detection runs in eboxdetect
		//		likemin			: detection likelihood in eboxdetect
		//


	SourceList(const std::string &eventListName,
			   const std::string &xAxis, const std::string &yAxis,
			   unsigned binX = 75, unsigned binY = 75,
			   double sourceRadius = 5, unsigned nruns = 1,
			   double likemin = 15.);
		//
		//	PURPOSE:
		//		Construct source list from an event list by
		//			a) accumulating an image with evselect
		//			b) running eboxdetect on the image
		//			c) show the image + found sources in Ds9
		//			d) let the user select a number of sources
		//


	SourceList(const std::string &eventListName,
			   const std::string &refSrcList,
			   const std::string &xAxis, const std::string &yAxis,
			   unsigned binX = 75, unsigned binY = 75,
			   double sourceRadius = 5.,
			   unsigned nruns = 1, double likemin = 15.);
		//
		//	PURPOSE:
		//		Construct source list from an event list by
		//			a) accumulating an image with evselect
		//			b) running eboxdetect on the image
		//			c) show the image + found sources in Ds9
		//			d) overlay position of sources in given reference source
		//			   on image
		//			e) let the user select a number of sources
		//


	const GeoRegion &region() const { return _region; }

	const std::string &eventListName() const { return _eventListName; }
	const std::string &xAxis() const { return _xAxis; }
	const std::string &yAxis() const { return _yAxis; }
	unsigned binX() const { return _binX; }
	unsigned binY() const { return _binY; }
	double sourceRadius() const { return _sourceRadius; }
	unsigned nruns() const { return _nruns; }
	double likemin() const { return _likemin; }

	~SourceList();

private:
	std::string		_eventListName;
	std::string		_xAxis, _yAxis;
	unsigned		_binX, _binY;
	double			_minX, _minY;
    double			_sourceRadius;
    unsigned		_nruns;
	double			_likemin;
	GeoRegion		_region;
	void construct(Table *sourceTable, double sourceRadius,
				   bool includeSources = true, bool withlable= true,
				   ShapeBasics::ShapeColor = ShapeBasics::WHITE);
	void completeSourceList(const string &, const string &);
	void diagnostic() const;
};

std::ostream &operator<<(std::ostream &os, const SourceList &sl);

#endif


