//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1997 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	srcdisplay.cc --- show result of source detection
//
//	DESCRIPTION:
//		srcdisplay visiualizes the result of a source detection run;
//		it displays the image on which the source detection was performed
//		and marks the position of each source with a region marker
//
//	CHANGE HISTORY:
//		UL, 1999-01-11: 1.0.0: - first version
//		HS, 2000-05-16: 0.1  : - Added further parameters, withregionfile and regionfile
//                               to allow regionfile to be exported.
//                               renamed tool from showdetect to task srcdisplay-0.1
//
//
//	AUTHOR:
//		U. Lammers, Mon Nov  1 12:10:33 MET 1999
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /home/sasbuild/repositories/sasdev/sas/packages/srcdisplay/src/srcdisplay.cc,v 1.8 2003/05/23 15:39:02 ulammers Exp $
//
/* C++ */
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <cstdio>	//	tmpnam()
#include <unistd.h>

/* SAS */
#include "Dal.h"
#include "param.h"
#include "ParamInfoSet.h"
#include "errstr.h"
#include "Region.h"
#include "imgdisplay.h"
#include "Ds9.h"

/* Local */
#include "SourceList.h"
#include "srcdisplay.h"

using namespace std;

void srcdisplay()
{
	//	Check if input image has WCS coords. If not, overlayed
	//	source positions may be incorrect.
	bool withImageSet = booleanParameter("withimageset");
	string imageSet = stringParameter("imageset");
	DataSet * set = dataSetServer->open(imageSet, Dal::Read);
	
	//	Look at first array in dataset. Could have forced input
	//	image to be of type 'array', but this may
	//	cause incompatibilities with imgdisplay
	Array * array = set->array(0);
	if (array->hasAttribute("CDELT1") && array->hasAttribute("CDELT2")) {
		string ctype1 = array->attribute("CTYPE1")->asString();
		string ctype2 = array->attribute("CTYPE2")->asString();
		if (ctype1.find("RA") != 0 || ctype2.find("DEC") != 0)
			errstr
			  <<"The WCS axes types ["
			  <<ctype1.c_str()
			  <<","
			  <<ctype2.c_str()
			  <<"] do not correspond to an equatorial system projection.  "
				"The displayed source positions may therefore be incorrect."
			  << Sas::warning("invalidWCS");
	} else
		errstr<<"The input array does not contain any WCS information. "
				"The displayed source positions may therefore be incorrect."
			  <<Sas::warning("invalidWCS");

	SourceList sl(stringParameter("boxlistset"),
				  realParameter("sourceradius"),
				  booleanParameter("includesources"),
				  booleanParameter("uselabel"),
				  ShapeBasics::toColor(stringParameter("srccolor")));
	
	bool withregionfile = booleanParameter("withregionfile");
	const char * regionFile = withregionfile
									? stringParameter("regionfile").c_str()
									: tmpnam(0);

	ofstream of(regionFile);
	if (!of)
		errstr<<"Cannot open region file `"<<regionFile<<"' for write"
			  <<Sas::error("UnableToCreateFile");

	of<<ImageDisplayer::Region(sl.region(), ImageDisplayer::FK5)<<endl;
	of.close();

	//	display image with markers
	Info::Task *imgdisplay = task("imgdisplay");
	imgdisplay->setParameter("withimagefile", withImageSet);
	if (withImageSet)
		imgdisplay->setParameter("imagefile", imageSet);
	imgdisplay->setParameter("withcommands", true);
	vector<string> commands;
	if (!booleanParameter("overlay")) {
		commands.push_back("regions deleteall");
		commands.push_back("zoom to 1");
	}
	stringstream s; s<<"regions file "<<regionFile;
	commands.push_back(s.str());
	imgdisplay->setParameter("commands", commands);
	imgdisplay->exec();

	if (!withregionfile)
		unlink(regionFile);
} // srcdisplay()
